package com.huashe.citybrain.commons.base.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

/**
 * $字节数组转换工具
 *
 * @author chen.jinshu (青禾) 2019/03/01
 */
public abstract class ByteArrayConverterUtils {
    private static final Logger log = LoggerFactory.getLogger(ByteArrayConverterUtils.class);

    private static final int BYTE_TYPE_BYTE_COUNT = 1;

    private static final int SHORT_TYPE_BYTE_COUNT = 2;

    private static final int INT_TYPE_BYTE_COUNT = 4;

    private static final int LONG_TYPE_BYTE_COUNT = 8;

    /**
     * $把字节数组转为字符串
     *
     * @param data 字节数组
     * @return String
     */
    public static String byteArrayToString(byte[] data) {
        StringBuilder sb = new StringBuilder();
        for (byte b : data) {
            sb.append(String.format("%02X", b));
        }
        return sb.toString();
    }

    /**
     * $把字节数组转换成double类型
     *
     * @param b 字节数组
     * @return double
     */
    public static double byteArrayToDouble(byte[] b) {
        long l;
        l = b[0];
        l &= 0xFFL;
        l |= ((long) b[1] << 8);
        l &= 0xFFFFL;
        l |= ((long) b[2] << 16);
        l &= 0xFFFFFFL;
        l |= ((long) b[3] << 24);
        l &= 0xFFFFFFFFL;
        l |= ((long) b[4] << 32);
        l &= 0xFFFFFFFFFFL;
        l |= ((long) b[5] << 40);
        l &= 0xFFFFFFFFFFFFL;
        l |= ((long) b[6] << 48);
        l &= 0xFFFFFFFFFFFFFFL;
        l |= ((long) b[7] << 56);
        return Double.longBitsToDouble(l);
    }

    /**
     * $短整形数字转换成字节数组
     *
     * @param n 源数字
     * @return byte[]
     */
    public static byte[] shortToByteArray(short n) {
        byte[] b = new byte[2];
        b[1] = (byte) (n & 0xff);
        b[0] = (byte) ((n >> 8) & 0xff);
        return b;
    }

    /**
     * $把int型数字转换成字节数组
     *
     * @param n 源数字
     * @return byte[]
     */
    public static byte[] intToByteArray(int n) {
        byte[] b = new byte[4];
        b[3] = (byte) (n & 0xff);
        b[2] = (byte) (n >> 8 & 0xff);
        b[1] = (byte) (n >> 16 & 0xff);
        b[0] = (byte) (n >> 24 & 0xff);
        return b;
    }

    /**
     * $把字节数组转换成long类型
     *
     * @param bytes 字节数组
     * @return long
     */
    public static long byteArrayToLong(byte[] bytes) {
        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
        DataInputStream dis = new DataInputStream(bais);
        long result = 0;
        try {
            int len = dis.available();
            if (len == BYTE_TYPE_BYTE_COUNT) {
                result = dis.readByte();
            }
            else if (len == SHORT_TYPE_BYTE_COUNT) {
                result = dis.readShort();
            }
            else if (len == INT_TYPE_BYTE_COUNT) {
                result = dis.readInt();
            }
            else if (len == LONG_TYPE_BYTE_COUNT) {
                result = dis.readLong();
            }
        }
        catch (IOException e) {
            log.error("fail!", e);
        }
        finally {
            try {
                dis.close();
                bais.close();
            }
            catch (IOException e) {
                log.error("fail!", e);
            }
        }
        return result;
    }

    /**
     * $把对象转化为字节数组
     *
     * @param obj 源对象
     * @return byte[]
     */
    public static byte[] objectToByteArray(Object obj) {
        byte[] bytes = null;
        ByteArrayOutputStream bo = new ByteArrayOutputStream();
        ObjectOutputStream oo = null;

        try {
            oo = new ObjectOutputStream(bo);
            oo.writeObject(obj);
            bytes = bo.toByteArray();
        }
        catch (Exception e) {
            log.error("fail!", e);
        }
        finally {
            CloseUtils.close(bo);
            CloseUtils.close(oo);
        }

        return (bytes);
    }
}