/*
 * 文 件 名:  ResponseUtils
 * 版    权:  浩鲸云计算科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  learshaw
 * 修改时间:  2019/8/27
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.citybrain.commons.base.utils;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.huashe.citybrain.commons.base.bean.Response;
import com.huashe.citybrain.commons.base.exception.Assert;
import com.huashe.citybrain.commons.base.exception.BusinessException;
import com.huashe.citybrain.commons.base.exception.ErrorCode;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ClassUtils;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 测试响应工具类
 * <功能详细描述>
 *
 * @author learshaw
 * @version [版本号, 2019/8/27]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class TestUtils {
    private static Logger log = LoggerFactory.getLogger(TestUtils.class);

    /**
     * 获取对象列表
     *
     * @param t     对象
     * @param cType 类型
     * @param <T>   响应
     * @return Response
     */
    public static <T> Response<T> getResponse(T t, Class<T> cType) {
        Response<T> response = new Response<>();

        response.setCode(0);
        response.setMessage("成功!");
        response.setData(t);

        return response;
    }

    public static File getTestFile(String filePath) {
        String path = ClassUtils.getDefaultClassLoader().getResource("").getPath() + filePath;
        return new File(path);
    }

    /**
     * 获取对象列表
     *
     * @param file  测试文件
     * @param cType 类型
     * @param <T>   响应
     * @return Response
     */
    public static <T> Response<T> getResponseObj(File file, Class<T> cType) {
        Response<T> response = new Response<>();

        try {
            T obj = readObj(file, cType);
            response.setData(obj);
        }
        catch (BusinessException e) {
            CommonUtils.businessExceptionRes(response, e);
            log.error("getResponseList fail!", e);
        }
        catch (Exception e) {
            log.error("getResponseList fail!", e);
            CommonUtils.unknownExceptionRes(response, e);
        }

        return response;
    }

    /**
     * 获取对象列表
     *
     * @param file  测试文件
     * @param cType 类型
     * @param <T>   响应
     * @return Response
     */
    public static <T> Response<List<T>> getResponseList(File file, Class<T> cType) {
        Response<List<T>> response = new Response<>();

        try {
            List<T> list = readList(file, cType);
            response.setData(list);
        }
        catch (BusinessException e) {
            CommonUtils.businessExceptionRes(response, e);
            log.error("getResponseList fail!", e);
        }
        catch (Exception e) {
            log.error("getResponseList fail!", e);
            CommonUtils.unknownExceptionRes(response, e);
        }

        return response;
    }

    /**
     * 获取Map响应
     *
     * @param file file
     * @return Map
     */
    public static JSONObject readJSON(File file) {
        Assert.notNull(file, ErrorCode.UNKNOWN_ERROR);
        Assert.isTrue(file.exists(), ErrorCode.UNKNOWN_ERROR.getCode(), "测试参数文件不存在！");
        return readObj(file, JSONObject.class);
    }

    /**
     * Map转换
     *
     * @return 响应
     */
    public static Map<String, Long> convertLongMap(Map source) {
        Map<String, Long> dataMap = new HashMap<>();
        Set<Map.Entry<String, Object>> tmpSet = source.entrySet();
        Iterator<Map.Entry<String, Object>> iterator = tmpSet.iterator();

        while (iterator.hasNext()) {
            Map.Entry<String, Object> entry = iterator.next();
            String key = entry.getKey();
            Object value = entry.getValue();
            dataMap.put(key, Long.valueOf(value.toString()));
        }

        return dataMap;
    }

    /**
     * Map转换
     *
     * @return 响应
     */
    public static Map<String, String> convertStringMap(Map<String, Object> source) {
        Map<String, String> dataMap = new HashMap<>();
        Set<Map.Entry<String, Object>> tmpSet = source.entrySet();
        Iterator<Map.Entry<String, Object>> iterator = tmpSet.iterator();

        while (iterator.hasNext()) {
            Map.Entry<String, Object> entry = iterator.next();
            String key = entry.getKey();
            Object value = entry.getValue();
            dataMap.put(key, value.toString());
        }

        return dataMap;
    }

    /**
     * 获取对象
     *
     * @param file  文件
     * @param cType 类型
     * @param <T>
     * @return 对象
     */
    public static <T> T readObj(File file, Class<T> cType) {
        T t = null;

        try {
            String dataStr = FileUtils.readFileToString(file, "UTF-8");
            t = JSON.parseObject(dataStr, cType);
        }
        catch (Exception e) {
            log.error("readFileTo Obj fail!", e);
        }

        return t;
    }

    /**
     * 获取对象列表
     *
     * @param file  文件
     * @param cType 类型
     * @param <T>   对象列表
     * @return List
     */
    public static <T> List<T> readList(File file, Class<T> cType) {
        List<T> list = null;

        try {
            String dataStr = FileUtils.readFileToString(file, "UTF-8");
            list = JSON.parseArray(dataStr, cType);
        }
        catch (Exception e) {
            log.error("readFileTo Obj fail!", e);
        }

        return list;
    }
}